# Rogue Observatory Coordinator

## Installation
The coordinator (or coordinator.exe on Windows) binary is fully self-contained
and can be copied to any appropriate location as desired.

### MongoDB
The coordinator requires access to a MongoDB instance for storing data. For
more information, see the Configuration section below.

## Usage
Some configuration details can be passed on the command line:
- `--help`: display help information
- `--version`: display version information and exit
- `--config`: path to a configuration file (see Configuration section below)
- `--port`: port to run the REST API on (default 13100)
- `--address`: advertised REST API address (default auto-detect)
  - *This is only the address that will be sent to other nodes as the canonical
  address for this node.*
- `--peers`: comma-separated list of other coordinator node endpoints to connect
to when bootstrapping (see Peering section below)

### Configuration
The configuration file can provide many of the same options, and a broad set of
additional settings. *Any settings provided on the command line will override the
equivalent seting in the configuration file.* The configuration is a text file in JSON format:
```javascript
{
	"Port": 13100,
	"Address": "192.168.13.1",
	"AgentUpdateInterval": 15,
	"PeerUpdateInterval": 15,
	"PeerCheckInterval": 5,
	"MongoHost": "localhost",
	"MongoDatabase": "Observatory",
	"BootstrapPeers": [],
	"UIFilePath": "/opt/observeratory/ui",
	"UIWebPath": "ui",
	"UIPort": 8000
}
```

The available settings are:
- `Port`: integer port to expose the REST API on (same as the `-port` flag)
- `Address`: address to advertise (same as the `-address` flag)
- `AgentUpdateInterval`: time (in seconds) between agent check-ins (default `30`)
- `PeerUpdateInterval`: time (in seconds) between querying peers for their peers
to update the peer list (default `30`)
- `PeerCheckInterval`: time (in seconds) between checking if peer coordinators
are up (default `5`)
- `UIFilePath`: path to a directory containing the Rogue Observatory Web UI files
(optional, see UI Serving below)
- `UIWebPath`: path to expose the web UI on (optional, see UI Serving below)
- `UIPort`: port to expose the web UI on (optional, see UI Serving below)
- `MongoHost`: address for the MongoDB server (default `"localhost"`)
- `MongoDatabase`: the database name to use (default `"Observatory"`)
- `MongoUser`: username to authenticate with MongoDB, if any
- `MongoPassword`: password to authenticate with MongoDB, if any
- `BootstrapPeers`: peers to connect to at startup (same as the `-peers` flag)
- `SMTPHost`: SMTP host address for sending alert e-mails (optional)
- `SMTPPort`: SMTP host port (optional)
- `SMTPUser`: username for authenticating with the SMTP server, if any
- `SMTPPassword`: password for authenticating with the SMTP server, if any
- `EmailFrom`: "from" address to use for alert e-mails (optional)

### UI Serving
The Rogue Observatory Web UI is a set of static files, and so can be served by
any HTTP server desired - nginx, apache, IIS, etc. The coordinator is also
capable of serving the UI itself if desired, by one (or both) of two methods:
- Serve the UI on the same port as the REST API, under a given root path (e.g.
	`http://localhost:13100/ui`).
- Serve the UI on a different port, at the root path (e.g. `http://localhost:8000/`)

When the UI is hosted outside of the coordinator, it is important to provide a
`config.js` file in the root directory of the UI containing the following:
```javascript
var endpoint = "http://localhost:13100/";
```

Where the `endpoint` variable value is an appropriate address for reaching a
coordinator node.

### Expiring Data
Currently, the record of every executed check is retained indefinitely. You can,
however, control the data set size by adding a ttl index directly in MongoDB:
```javascript
db.CheckResults.createindex({"time" : -1}, {
    "name":"ttl",
    "expireAfterSeconds":604800
})
```

This example uses a retention of 604800 seconds, or 1 week.
